///////////////////////////////////////////////
//    FILE: defsim.h
//          class definitions for mass-spring networks
///////////////////////////////////////////////

#include "lib/lib_defs.h"

class NETWORK {
private:
  class MASS *masses;
  class SPRING *springs;
public:
  void print(void);
  void draw(void);
  void add_mass(int argc, char **argv);
  void add_spring(int argc, char **argv);
  void fix(int);
  void sim();
  void integ();
  class MASS *find_mass(int);

  friend void user_loop(void);
};

class MASS {
private:
  int id;             // id number
  float mass;         // mass
  FVEC posn;		  // position
  FVEC vel;           // velocity
  FVEC acc;           // acceleration

  FVEC prevposn;      // position at previous iteration
  FVEC prevvel;	      // velocity at previous iteration
  FVEC force;		  // total force being exerted on this mass

  int type;           // fixed or free?
  int nsprings;       // number of connected springs
  class SLIST *slist; // list of springs					 
  class MASS *next;   // next in linked list						   

  bool Flag;		  // general purpose flag/marker
  float attraction;	  // attraction/repulsion coefficient

public:
  MASS(int id, float mass, float *posn, float *vel, float attr);
  void print();       // print contents of MASS structure
  void inverseDynamics( FVEC newpos );
  friend class SLIST;
  friend class NETWORK;
  friend class SPRING;            // needs access to mass id

  friend void user_loop(void);
};

class SPRING {
private:
  float kp;           // spring constant
  float kd;           // damper constant					  
  float len;          // spring rest length						   
  class MASS *m1,*m2; // associated masses
  class SPRING *next;
  bool fVis;		  // is spring visible (good for hiding flexion sprngs)
public:
  SPRING(MASS *, MASS *, float kp, float kd, float len, bool fVisible);
  void print();       // prints structure
  friend class NETWORK;						   
  friend class MASS;
};

class SLIST {
private:
  class SPRING *spring;
  class SLIST *next;
public:
  friend class SPRING;
  friend class NETWORK;
  friend class MASS;
};


void draw_tetra();
void make_sm();
float dist2ground( FVEC v );

extern NETWORK Network;

const int FIXED = 1;
const int FREE = 0;

extern FVEC grnd[4];
extern FVEC plane;
extern bool fRecordOn;

