/************************************************************
	FILE:		user.C
	CONTAINS:	user functions
************************************************************/

#include "tcl.h"
#include "glaux.h"
#include <strings.h>
#include <unistd.h>

#include "lib/lib_defs.h"
#include "lib/lib_globals.h"
#include "lib/lib_draw.h"
#include "lib/lib_user.h"

#include "user.h"
#include "defsim.h"

int SIm_flag  = 0;
double Sim_time   = 0.0;
double DTsim  = 0.01;
double DTdisp = 0.05;
double Disp_time = 0.0;

/************************************************************
	PROC:	user_init(void)
	DOES:	Initializes user-defined globals
************************************************************/

void user_init(void)
{
  extern Tcl_Interp *Interp;


	Tcl_CreateCommand(Interp, "test", test, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "masspt", masspt, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "spring", spring, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "net", net, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "fix", fix, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "wind", wind, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "ground", ground, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "cloth_model", cloth_model, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "dt_sim", dt_sim, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "dt_disp", dt_disp, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "record_on", record_on, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "drag_coeffs", drag_coeffs, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "bounce_alpha", bounce_alpha, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(Interp, "friction_kcoeff", friction_kcoeff, (ClientData) NULL,
					  (Tcl_CmdDeleteProc *) NULL);
    Tcl_LinkVar(Interp, "simflag", (char *) &SIm_flag, TCL_LINK_INT);
    Tcl_LinkVar(Interp, "dtsim", (char *) &DTsim, TCL_LINK_DOUBLE);
    Tcl_LinkVar(Interp, "dtdisp", (char *) &DTdisp, TCL_LINK_DOUBLE);
}

/************************************************************
	PROC:	user_loop(void)
	DOES:	All the action happens here. 
            Queries for both OpenGL and TK events
************************************************************/

void user_loop(void)
{
  extern int Quit;
  extern void check_events(void);
  extern Tcl_Interp *Interp;
  STR string;
  STR command;
  int count = 0;

  while (1) {
//	if (Interp) {
//		sprintf(string, "%d", Counter);
//		Tcl_SetVar(Interp, "counter",string,0);
//	}
	if (SIm_flag) {         // should we be simulating?
	  Network.sim();        // calculate accelerations
	  Network.integ();      // integrate state to update posn, vel
	  Sim_time += DTsim;    // update simulation timestep
	  if (Sim_time > Disp_time) {     // time for redraw?
		display();        
		Disp_time += DTdisp;
		if( fRecordOn ) {
			sprintf(command,"img_save /local/tmp/images/img%d.ppm",count);
			count++;
			Tcl_Eval(Interp,command);
		}
	  }
	}
//	usleep(10);
	check_events();   // checks for Tcl/Tk events and events in OpenGL win
  }
}

/************************************************************
	PROC:	user_draw(void)
	DOES:	user's drawing code
************************************************************/

void user_draw(void)
{
  //  draw_wire_cube();
  Network.draw();
}

void draw_wire_cube( void )
{
   /* X faces */
   glColor3f( 1.0, 0.0, 0.0 );
   glBegin( GL_LINE_LOOP );
   glVertex3f( 1.0, 1.0, 1.0 );
   glVertex3f( 1.0, -1.0, 1.0 );
   glVertex3f( 1.0, -1.0, -1.0 );
   glVertex3f( 1.0, 1.0, -1.0 );
   glEnd();

   glColor3f( 0.0, 0.0, 1.0 );
   glBegin( GL_LINE_LOOP );
   glVertex3f( -1.0, 1.0, 1.0 );
   glVertex3f( -1.0, 1.0, -1.0 );
   glVertex3f( -1.0, -1.0, -1.0 );
   glVertex3f( -1.0, -1.0, 1.0 );
   glEnd();

   /* remaining lines */
   glColor3f( 0.0, 1.0, 0.0 );
   glBegin( GL_LINES );
   glVertex3f(  1.0, 1.0, -1.0 );
   glVertex3f( -1.0, 1.0, -1.0 );
   glVertex3f(  1.0, -1.0,  1.0 );
   glVertex3f( -1.0, -1.0,  1.0 );
   glVertex3f(  1.0,  1.0,  1.0 );
   glVertex3f( -1.0,  1.0,  1.0 );
   glVertex3f(  1.0,-1.0, -1.0 );
   glVertex3f( -1.0,-1.0, -1.0 );
   glEnd();
}

//////////////////////////////////////////////////
//	PROC:	user_gl_init(void)
//	DOES:	Initializes and OpenGL event handling
//          Probably not too useful, but you never know...
//////////////////////////////////////////////////

void user_gl_init(void)
{
  void key_toggle_view();       // defined later in this file

      // when V-key is pressed in OpenGL window,
      // call key_toggle_view()
  auxKeyFunc(AUX_v, key_toggle_view);
}

/*********************************************************
    PROC: key_toggle_view()
    DOES: toggles between rotate & translate modes
**********************************************************/

void key_toggle_view()
{
  switch(Mode) {
    case TRANSLATE_MODE:
      Mode = ROTATE_MODE;
      break;
    case ROTATE_MODE:
      Mode = TRANSLATE_MODE;
      break;
	}
  display();
}

///////////////////////////////////////////////////////////
//    PROC: test()
//    DOES: example of how to bind a new tcl function
///////////////////////////////////////////////////////////

int test(ClientData clientData,
		   Tcl_Interp *interp,
		   int argc, char **argv)
{
  printf("test()\n");
  return TCL_OK;
}

//////////////////////////////////////////////////
//    PROC:   parse
//    INPUT:  string, parselist, argc, argv
//    DOES:   parses items in a string, places results in args
//////////////////////////////////////////////////

parse(char *line, char *parselist, int *argc, STR argv[])
{
    int indx=0;
    int copying = 0;
    int narg=0;
    int argch=0;
    char    ch;

    while (line[indx]) {
        ch = line[indx++];
        if (ch=='#') break;
        if ( strchr(parselist,ch)) {
            if (copying) {
                copying=0;
                argv[narg][argch]=0;
                narg++;
                argch=0;
            }
        } else {
            if (!copying) copying=1;
            argv[narg][argch++] = ch;
        }
    }
    if (copying) {
        argv[narg][argch] = 0;
        *argc = ++narg;
    } else *argc = narg;
}
